@@ -2,11 +2,11 @@
  *  yosys -- Yosys Open SYnthesis Suite
  *
  *  Copyright (C) 2012  Clifford Wolf <clifford@clifford.at>
- *  
+ *
  *  Permission to use, copy, modify, and/or distribute this software for any
  *  purpose with or without fee is hereby granted, provided that the above
  *  copyright notice and this permission notice appear in all copies.
- *  
+ *
  *  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  *  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  *  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
@@ -37,14 +37,17 @@ struct BlifDumperConfig
 	bool conn_mode;
 	bool impltf_mode;
 	bool gates_mode;
+	bool cname_mode;
 	bool param_mode;
+	bool attr_mode;
 	bool blackbox_mode;
 
 	std::string buf_type, buf_in, buf_out;
 	std::map<RTLIL::IdString, std::pair<RTLIL::IdString, RTLIL::IdString>> unbuf_types;
 	std::string true_type, true_out, false_type, false_out, undef_type, undef_out;
 
-	BlifDumperConfig() : icells_mode(false), conn_mode(false), impltf_mode(false), gates_mode(false), param_mode(false), blackbox_mode(false) { }
+	BlifDumperConfig() : icells_mode(false), conn_mode(false), impltf_mode(false), gates_mode(false),
+			cname_mode(false), param_mode(false), attr_mode(false), blackbox_mode(false) { }
 };
 
 struct BlifDumper
@@ -55,12 +58,34 @@ struct BlifDumper
 	BlifDumperConfig *config;
 	CellTypes ct;
 
+	SigMap sigmap;
+	dict<SigBit, int> init_bits;
+
 	BlifDumper(std::ostream &f, RTLIL::Module *module, RTLIL::Design *design, BlifDumperConfig *config) :
-			f(f), module(module), design(design), config(config), ct(design)
+			f(f), module(module), design(design), config(config), ct(design), sigmap(module)
 	{
+		for (Wire *wire : module->wires())
+			if (wire->attributes.count("\\init")) {
+				SigSpec initsig = sigmap(wire);
+				Const initval = wire->attributes.at("\\init");
+				for (int i = 0; i < GetSize(initsig) && i < GetSize(initval); i++)
+					switch (initval[i]) {
+						case State::S0:
+							init_bits[initsig[i]] = 0;
+							break;
+						case State::S1:
+							init_bits[initsig[i]] = 1;
+							break;
+						case State::Sx:
+							init_bits[initsig[i]] = 2;
+							break;
+						default:
+							break;
+					}
+			}
 	}
 
-	std::vector<std::string> cstr_buf;
+	vector<shared_str> cstr_buf;
 
 	const char *cstr(RTLIL::IdString id)
 	{
@@ -92,6 +117,19 @@ struct BlifDumper
 		return cstr_buf.back().c_str();
 	}
 
+	const char *cstr_init(RTLIL::SigBit sig)
+	{
+		sigmap.apply(sig);
+
+		if (init_bits.count(sig) == 0)
+			return "";
+
+		string str = stringf(" %d", init_bits.at(sig));
+
+		cstr_buf.push_back(str);
+		return cstr_buf.back().c_str();
+	}
+
 	const char *subckt_or_gate(std::string cell_type)
 	{
 		if (!config->gates_mode)
@@ -103,6 +141,26 @@ struct BlifDumper
 		return "subckt";
 	}
 
+	void dump_params(const char *command, dict<IdString, Const> &params)
+	{
+		for (auto &param : params) {
+			f << stringf("%s %s ", command, RTLIL::id2cstr(param.first));
+			if (param.second.flags & RTLIL::CONST_FLAG_STRING) {
+				std::string str = param.second.decode_string();
+				f << stringf("\"");
+				for (char ch : str)
+					if (ch == '"' || ch == '\\')
+						f << stringf("\\%c", ch);
+					else if (ch < 32 || ch >= 127)
+						f << stringf("\\%03o", ch);
+					else
+						f << stringf("%c", ch);
+				f << stringf("\"\n");
+			} else
+				f << stringf("%s\n", param.second.as_string().c_str());
+		}
+	}
+
 	void dump()
 	{
 		f << stringf("\n");
@@ -196,6 +254,50 @@ struct BlifDumper
 				continue;
 			}
 
+			if (!config->icells_mode && cell->type == "$_NAND_") {
+				f << stringf(".names %s %s %s\n0- 1\n-0 1\n",
+						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")), cstr(cell->getPort("\\Y")));
+				continue;
+			}
+
+			if (!config->icells_mode && cell->type == "$_NOR_") {
+				f << stringf(".names %s %s %s\n00 1\n",
+						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")), cstr(cell->getPort("\\Y")));
+				continue;
+			}
+
+			if (!config->icells_mode && cell->type == "$_XNOR_") {
+				f << stringf(".names %s %s %s\n11 1\n00 1\n",
+						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")), cstr(cell->getPort("\\Y")));
+				continue;
+			}
+
+			if (!config->icells_mode && cell->type == "$_AOI3_") {
+				f << stringf(".names %s %s %s %s\n-00 1\n0-0 1\n",
+						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")), cstr(cell->getPort("\\C")), cstr(cell->getPort("\\Y")));
+				continue;
+			}
+
+			if (!config->icells_mode && cell->type == "$_OAI3_") {
+				f << stringf(".names %s %s %s %s\n00- 1\n--0 1\n",
+						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")), cstr(cell->getPort("\\C")), cstr(cell->getPort("\\Y")));
+				continue;
+			}
+
+			if (!config->icells_mode && cell->type == "$_AOI4_") {
+				f << stringf(".names %s %s %s %s %s\n-0-0 1\n-00- 1\n0--0 1\n0-0- 1\n",
+						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")),
+						cstr(cell->getPort("\\C")), cstr(cell->getPort("\\D")), cstr(cell->getPort("\\Y")));
+				continue;
+			}
+
+			if (!config->icells_mode && cell->type == "$_OAI4_") {
+				f << stringf(".names %s %s %s %s %s\n00-- 1\n--00 1\n",
+						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")),
+						cstr(cell->getPort("\\C")), cstr(cell->getPort("\\D")), cstr(cell->getPort("\\Y")));
+				continue;
+			}
+
 			if (!config->icells_mode && cell->type == "$_MUX_") {
 				f << stringf(".names %s %s %s %s\n1-0 1\n-11 1\n",
 						cstr(cell->getPort("\\A")), cstr(cell->getPort("\\B")),
@@ -204,14 +306,14 @@ struct BlifDumper
 			}
 
 			if (!config->icells_mode && cell->type == "$_DFF_N_") {
-				f << stringf(".latch %s %s fe %s\n",
-						cstr(cell->getPort("\\D")), cstr(cell->getPort("\\Q")), cstr(cell->getPort("\\C")));
+				f << stringf(".latch %s %s fe %s%s\n", cstr(cell->getPort("\\D")), cstr(cell->getPort("\\Q")),
+						cstr(cell->getPort("\\C")), cstr_init(cell->getPort("\\Q")));
 				continue;
 			}
 
 			if (!config->icells_mode && cell->type == "$_DFF_P_") {
-				f << stringf(".latch %s %s re %s\n",
-						cstr(cell->getPort("\\D")), cstr(cell->getPort("\\Q")), cstr(cell->getPort("\\C")));
+				f << stringf(".latch %s %s re %s%s\n", cstr(cell->getPort("\\D")), cstr(cell->getPort("\\Q")),
+						cstr(cell->getPort("\\C")), cstr_init(cell->getPort("\\Q")));
 				continue;
 			}
 
@@ -249,23 +351,12 @@ struct BlifDumper
 			}
 			f << stringf("\n");
 
+			if (config->cname_mode)
+				f << stringf(".cname %s\n", cstr(cell->name));
+			if (config->attr_mode)
+				dump_params(".attr", cell->attributes);
 			if (config->param_mode)
-				for (auto &param : cell->parameters) {
-					f << stringf(".param %s ", RTLIL::id2cstr(param.first));
-					if (param.second.flags & RTLIL::CONST_FLAG_STRING) {
-						std::string str = param.second.decode_string();
-						f << stringf("\"");
-						for (char ch : str)
-							if (ch == '"' || ch == '\\')
-								f << stringf("\\%c", ch);
-							else if (ch < 32 || ch >= 127)
-								f << stringf("\\%03o", ch);
-							else
-								f << stringf("%c", ch);
-						f << stringf("\"\n");
-					} else
-						f << stringf("%s\n", param.second.as_string().c_str());
-				}
+				dump_params(".param", cell->parameters);
 		}
 
 		for (auto &conn : module->connections())
@@ -333,8 +424,14 @@ struct BlifBackend : public Backend {
 		log("        do not generate buffers for connected wires. instead use the\n");
 		log("        non-standard .conn statement.\n");
 		log("\n");
+		log("    -attr\n");
+		log("        use the non-standard .attr statement to write cell attributes\n");
+		log("\n");
 		log("    -param\n");
-		log("        use the non-standard .param statement to write module parameters\n");
+		log("        use the non-standard .param statement to write cell parameters\n");
+		log("\n");
+		log("    -cname\n");
+		log("        use the non-standard .cname statement to write cell names\n");
 		log("\n");
 		log("    -blackbox\n");
 		log("        write blackbox cells with .blackbox statement.\n");
@@ -400,10 +497,18 @@ struct BlifBackend : public Backend {
 				config.conn_mode = true;
 				continue;
 			}
+			if (args[argidx] == "-cname") {
+				config.cname_mode = true;
+				continue;
+			}
 			if (args[argidx] == "-param") {
 				config.param_mode = true;
 				continue;
 			}
+			if (args[argidx] == "-attr") {
+				config.attr_mode = true;
+				continue;
+			}
 			if (args[argidx] == "-blackbox") {
 				config.blackbox_mode = true;
 				continue;
@@ -425,6 +530,7 @@ struct BlifBackend : public Backend {
 
 		std::vector<RTLIL::Module*> mod_list;
 
+		design->sort();
 		for (auto module_it : design->modules_)
 		{
 			RTLIL::Module *module = module_it.second;
