@@ -2,11 +2,11 @@
  *  yosys -- Yosys Open SYnthesis Suite
  *
  *  Copyright (C) 2012  Clifford Wolf <clifford@clifford.at>
- *  
+ *
  *  Permission to use, copy, modify, and/or distribute this software for any
  *  purpose with or without fee is hereby granted, provided that the above
  *  copyright notice and this permission notice appear in all copies.
- *  
+ *
  *  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  *  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  *  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
@@ -72,6 +72,53 @@ int getopt(int argc, char **argv, const char *optstring)
 
 USING_YOSYS_NAMESPACE
 
+#ifdef EMSCRIPTEN
+#  include <sys/stat.h>
+#  include <sys/types.h>
+
+extern "C" int main(int, char**);
+extern "C" void run(const char*);
+extern "C" const char *errmsg();
+extern "C" const char *prompt();
+
+int main(int, char**)
+{
+	mkdir("/work", 0777);
+	chdir("/work");
+	log_files.push_back(stdout);
+	log_error_stderr = true;
+	yosys_banner();
+	yosys_setup();
+}
+
+void run(const char *command)
+{
+	int selSize = GetSize(yosys_get_design()->selection_stack);
+	try {
+		log_last_error = "Internal error (see JavaScript console for details)";
+		run_pass(command);
+		log_last_error = "";
+	} catch (...) {
+		while (GetSize(yosys_get_design()->selection_stack) > selSize)
+			yosys_get_design()->selection_stack.pop_back();
+		throw;
+	}
+}
+
+const char *errmsg()
+{
+	return log_last_error.c_str();
+}
+
+const char *prompt()
+{
+	const char *p = create_prompt(yosys_get_design(), 0);
+	while (*p == '\n') p++;
+	return p;
+}
+
+#else /* EMSCRIPTEN */
+
 int main(int argc, char **argv)
 {
 	std::string frontend_command = "auto";
@@ -170,7 +217,7 @@ int main(int argc, char **argv)
 		printf("        print version information and exit\n");
 		printf("\n");
 		printf("The option -S is an shortcut for calling the \"synth\" command, a default\n");
-		printf("script for transforming the verilog input to a gate-level netlist. For example:\n");
+		printf("script for transforming the Verilog input to a gate-level netlist. For example:\n");
 		printf("\n");
 		printf("    yosys -o output.blif -S input.v\n");
 		printf("\n");
@@ -357,7 +404,7 @@ int main(int argc, char **argv)
 		log("%s\n", yosys_version_str);
 
 		int64_t total_ns = 0;
-		std::set<std::tuple<int64_t, int, std::string>> timedat;
+		std::set<tuple<int64_t, int, std::string>> timedat;
 
 		for (auto &it : pass_register)
 			if (it.second->call_counter) {
@@ -440,3 +487,5 @@ int main(int argc, char **argv)
 	return 0;
 }
 
+#endif /* EMSCRIPTEN */
+
