@@ -2,11 +2,11 @@
  *  yosys -- Yosys Open SYnthesis Suite
  *
  *  Copyright (C) 2012  Clifford Wolf <clifford@clifford.at>
- *  
+ *
  *  Permission to use, copy, modify, and/or distribute this software for any
  *  purpose with or without fee is hereby granted, provided that the above
  *  copyright notice and this permission notice appear in all copies.
- *  
+ *
  *  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  *  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  *  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
@@ -18,6 +18,8 @@
  */
 
 #include "kernel/yosys.h"
+#include "kernel/satgen.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -148,7 +150,7 @@ void Pass::call(RTLIL::Design *design, std::string command)
 	std::vector<std::string> args;
 
 	std::string cmd_buf = command;
-	std::string tok = next_token(cmd_buf, " \t\r\n");
+	std::string tok = next_token(cmd_buf, " \t\r\n", true);
 
 	if (tok.empty())
 		return;
@@ -199,7 +201,7 @@ void Pass::call(RTLIL::Design *design, std::string command)
 			call(design, args);
 			args.clear();
 		}
-		tok = next_token(cmd_buf, " \t\r\n");
+		tok = next_token(cmd_buf, " \t\r\n", true);
 	}
 
 	call(design, args);
@@ -207,7 +209,7 @@ void Pass::call(RTLIL::Design *design, std::string command)
 
 void Pass::call(RTLIL::Design *design, std::vector<std::string> args)
 {
-	if (args.size() == 0 || args[0][0] == '#')
+	if (args.size() == 0 || args[0][0] == '#' || args[0][0] == ':')
 		return;
 
 	if (echo_mode) {
@@ -357,8 +359,7 @@ void Frontend::extra_args(std::istream *&f, std::string &filename, std::vector<s
 			}
 			f = new std::istringstream(last_here_document);
 		} else {
-			if (filename.substr(0, 2) == "+/")
-				filename = proc_share_dirname() + filename.substr(1);
+			rewrite_filename(filename);
 			std::ifstream *ff = new std::ifstream;
 			ff->open(filename.c_str());
 			if (ff->fail())
@@ -533,23 +534,36 @@ void Backend::backend_call(RTLIL::Design *design, std::ostream *f, std::string f
 	design->check();
 }
 
+static struct CellHelpMessages {
+	dict<string, string> cell_help, cell_code;
+	CellHelpMessages() {
+#include "techlibs/common/simlib_help.inc"
+#include "techlibs/common/simcells_help.inc"
+		cell_help.sort();
+		cell_code.sort();
+	}
+} cell_help_messages;
+
 struct HelpPass : public Pass {
 	HelpPass() : Pass("help", "display help messages") { }
 	virtual void help()
 	{
 		log("\n");
-		log("    help  .............  list all commands\n");
-		log("    help <command>  ...  print help message for given command\n");
-		log("    help -all  ........  print complete command reference\n");
+		log("    help  ................  list all commands\n");
+		log("    help <command>  ......  print help message for given command\n");
+		log("    help -all  ...........  print complete command reference\n");
+		log("\n");
+		log("    help -cells ..........  list all cell types\n");
+		log("    help <celltype>  .....  print help message for given cell type\n");
+		log("    help <celltype>+  ....  print verilog code for given cell type\n");
 		log("\n");
 	}
 	void escape_tex(std::string &tex)
 	{
-		size_t pos = 0;
-		while ((pos = tex.find('_', pos)) != std::string::npos) {
+		for (size_t pos = 0; (pos = tex.find('_', pos)) != std::string::npos; pos += 2)
 			tex.replace(pos, 1, "\\_");
-			pos += 2;
-		}
+		for (size_t pos = 0; (pos = tex.find('$', pos)) != std::string::npos; pos += 2)
+			tex.replace(pos, 1, "\\$");
 	}
 	void write_tex(FILE *f, std::string cmd, std::string title, std::string text)
 	{
@@ -609,6 +623,7 @@ struct HelpPass : public Pass {
 				log("    %-20s %s\n", it.first.c_str(), it.second->short_help.c_str());
 			log("\n");
 			log("Type 'help <command>' for more information on a command.\n");
+			log("Type 'help -cells' for a list of all cell types.\n");
 			log("\n");
 			return;
 		}
@@ -624,6 +639,18 @@ struct HelpPass : public Pass {
 					it.second->help();
 				}
 			}
+			else if (args[1] == "-cells") {
+				log("\n");
+				for (auto &it : cell_help_messages.cell_help) {
+					string line = split_tokens(it.second, "\n").at(0);
+					string cell_name = next_token(line);
+					log("    %-15s %s\n", cell_name.c_str(), line.c_str());
+				}
+				log("\n");
+				log("Type 'help <cell_type>' for more information on a cell type.\n");
+				log("\n");
+				return;
+			}
 			// this option is undocumented as it is for internal use only
 			else if (args[1] == "-write-tex-command-reference-manual") {
 				FILE *f = fopen("command-reference-manual.tex", "wt");
@@ -649,17 +676,27 @@ struct HelpPass : public Pass {
 				}
 				fclose(f);
 			}
-			else if (pass_register.count(args[1]) == 0)
-				log("No such command: %s\n", args[1].c_str());
-			else
+			else if (pass_register.count(args[1])) {
 				pass_register.at(args[1])->help();
+			}
+			else if (cell_help_messages.cell_help.count(args[1])) {
+				log("%s", cell_help_messages.cell_help.at(args[1]).c_str());
+				log("Run 'help %s+' to display the Verilog model for this cell type.\n", args[1].c_str());
+				log("\n");
+			}
+			else if (cell_help_messages.cell_code.count(args[1])) {
+				log("\n");
+				log("%s", cell_help_messages.cell_code.at(args[1]).c_str());
+			}
+			else
+				log("No such command or cell type: %s\n", args[1].c_str());
 			return;
 		}
 
 		help();
 	}
 } HelpPass;
- 
+
 struct EchoPass : public Pass {
 	EchoPass() : Pass("echo", "turning echoing back of commands on and off") { }
 	virtual void help()
@@ -692,6 +729,18 @@ struct EchoPass : public Pass {
 		log("echo %s\n", echo_mode ? "on" : "off");
 	}
 } EchoPass;
- 
+
+SatSolver *yosys_satsolver_list;
+SatSolver *yosys_satsolver;
+
+struct MinisatSatSolver : public SatSolver {
+	MinisatSatSolver() : SatSolver("minisat") {
+		yosys_satsolver = this;
+	}
+	virtual ezSAT *create() YS_OVERRIDE {
+		return new ezMiniSAT();
+	}
+} MinisatSatSolver;
+
 YOSYS_NAMESPACE_END
 
