@@ -2,11 +2,11 @@
  *  yosys -- Yosys Open SYnthesis Suite
  *
  *  Copyright (C) 2012  Clifford Wolf <clifford@clifford.at>
- *  
+ *
  *  Permission to use, copy, modify, and/or distribute this software for any
  *  purpose with or without fee is hereby granted, provided that the above
  *  copyright notice and this permission notice appear in all copies.
- *  
+ *
  *  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  *  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  *  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
@@ -66,7 +66,7 @@ void generate(RTLIL::Design *design, const std::vector<std::string> &celltypes,
 				for (auto &conn : i2.second->connections()) {
 					if (conn.first[0] != '$')
 						portnames.insert(conn.first);
-					portwidths[conn.first] = std::max(portwidths[conn.first], conn.second.size());
+					portwidths[conn.first] = max(portwidths[conn.first], conn.second.size());
 				}
 				for (auto &para : i2.second->parameters)
 					parameters.insert(para.first);
@@ -84,8 +84,8 @@ void generate(RTLIL::Design *design, const std::vector<std::string> &celltypes,
 
 		for (auto &decl : portdecls)
 			if (decl.index > 0) {
-				portwidths[decl.portname] = std::max(portwidths[decl.portname], 1);
-				portwidths[decl.portname] = std::max(portwidths[decl.portname], portwidths[stringf("$%d", decl.index)]);
+				portwidths[decl.portname] = max(portwidths[decl.portname], 1);
+				portwidths[decl.portname] = max(portwidths[decl.portname], portwidths[stringf("$%d", decl.index)]);
 				log("  port %d: %s [%d:0] %s\n", decl.index, decl.input ? decl.output ? "inout" : "input" : "output", portwidths[decl.portname]-1, RTLIL::id2cstr(decl.portname));
 				if (indices.count(decl.index) > ports.size())
 					log_error("Port index (%d) exceeds number of found ports (%d).\n", decl.index, int(ports.size()));
@@ -106,7 +106,7 @@ void generate(RTLIL::Design *design, const std::vector<std::string> &celltypes,
 					log_assert(!indices.empty());
 					indices.erase(d.index);
 					ports[d.index-1] = d;
-					portwidths[d.portname] = std::max(portwidths[d.portname], 1);
+					portwidths[d.portname] = max(portwidths[d.portname], 1);
 					log("  port %d: %s [%d:0] %s\n", d.index, d.input ? d.output ? "inout" : "input" : "output", portwidths[d.portname]-1, RTLIL::id2cstr(d.portname));
 					goto found_matching_decl;
 				}
@@ -268,7 +268,7 @@ void hierarchy_worker(RTLIL::Design *design, std::set<RTLIL::Module*> &used, RTL
 
 	if (indent == 0)
 		log("Top module:  %s\n", mod->name.c_str());
-	else
+	else if (!mod->get_bool_attribute("\\blackbox"))
 		log("Used module: %*s%s\n", indent, "", mod->name.c_str());
 	used.insert(mod);
 
@@ -285,7 +285,7 @@ void hierarchy_worker(RTLIL::Design *design, std::set<RTLIL::Module*> &used, RTL
 	}
 }
 
-void hierarchy(RTLIL::Design *design, RTLIL::Module *top, bool purge_lib, bool first_pass)
+void hierarchy_clean(RTLIL::Design *design, RTLIL::Module *top, bool purge_lib)
 {
 	std::set<RTLIL::Module*> used;
 	hierarchy_worker(design, used, top, 0);
@@ -295,17 +295,19 @@ void hierarchy(RTLIL::Design *design, RTLIL::Module *top, bool purge_lib, bool f
 		if (used.count(it.second) == 0)
 			del_modules.push_back(it.second);
 
+	int del_counter = 0;
 	for (auto mod : del_modules) {
-		if (first_pass && mod->name.substr(0, 9) == "$abstract")
+		if (mod->name.substr(0, 9) == "$abstract")
 			continue;
 		if (!purge_lib && mod->get_bool_attribute("\\blackbox"))
 			continue;
 		log("Removing unused module `%s'.\n", mod->name.c_str());
 		design->modules_.erase(mod->name);
+		del_counter++;
 		delete mod;
 	}
 
-	log("Removed %d unused modules.\n", GetSize(del_modules));
+	log("Removed %d unused modules.\n", del_counter);
 }
 
 bool set_keep_assert(std::map<RTLIL::Module*, bool> &cache, RTLIL::Module *mod)
@@ -313,12 +315,23 @@ bool set_keep_assert(std::map<RTLIL::Module*, bool> &cache, RTLIL::Module *mod)
 	if (cache.count(mod) == 0)
 		for (auto c : mod->cells()) {
 			RTLIL::Module *m = mod->design->module(c->type);
-			if ((m != nullptr && set_keep_assert(cache, m)) || c->type == "$assert")
+			if ((m != nullptr && set_keep_assert(cache, m)) || c->type.in("$assert", "$assume"))
 				return cache[mod] = true;
 		}
 	return cache[mod];
 }
 
+int find_top_mod_score(Design *design, Module *module, dict<Module*, int> &db)
+{
+	if (db.count(module) == 0) {
+		db[module] = 0;
+		for (auto cell : module->cells())
+			if (design->module(cell->type))
+				db[module] = max(db[module], find_top_mod_score(design, design->module(cell->type), db) + 1);
+	}
+	return db.at(module);
+}
+
 struct HierarchyPass : public Pass {
 	HierarchyPass() : Pass("hierarchy", "check, expand and clean up design hierarchy") { }
 	virtual void help()
@@ -339,7 +352,7 @@ struct HierarchyPass : public Pass {
 		log("\n");
 		log("    -purge_lib\n");
 		log("        by default the hierarchy command will not remove library (blackbox)\n");
-		log("        module. use this options to also remove unused blackbox modules.\n");
+		log("        modules. use this option to also remove unused blackbox modules.\n");
 		log("\n");
 		log("    -libdir <directory>\n");
 		log("        search for files named <module_name>.v in the specified directory\n");
@@ -363,6 +376,9 @@ struct HierarchyPass : public Pass {
 		log("        specified top module. otherwise a module with the 'top' attribute set\n");
 		log("        will implicitly be used as top module, if such a module exists.\n");
 		log("\n");
+		log("    -auto-top\n");
+		log("        automatically determine the top of the design hierarchy and mark it.\n");
+		log("\n");
 		log("In -generate mode this pass generates blackbox modules for the given cell\n");
 		log("types (wildcards supported). For this the design is searched for cells that\n");
 		log("match the given types and then the given port declarations are used to\n");
@@ -372,7 +388,7 @@ struct HierarchyPass : public Pass {
 		log("\n");
 		log("Input ports are specified with the 'i' prefix, output ports with the 'o'\n");
 		log("prefix and inout ports with the 'io' prefix. The optional <num> specifies\n");
-		log("the position of the port in the parameter list (needed when instanciated\n");
+		log("the position of the port in the parameter list (needed when instantiated\n");
 		log("using positional arguments). When <num> is not specified, the <portname> can\n");
 		log("also contain wildcard characters.\n");
 		log("\n");
@@ -389,6 +405,7 @@ struct HierarchyPass : public Pass {
 		RTLIL::Module *top_mod = NULL;
 		std::vector<std::string> libdirs;
 
+		bool auto_top_mode = false;
 		bool generate_mode = false;
 		bool keep_positionals = false;
 		bool nokeep_asserts = false;
@@ -470,6 +487,10 @@ struct HierarchyPass : public Pass {
 					log_cmd_error("Module `%s' not found!\n", args[argidx].c_str());
 				continue;
 			}
+			if (args[argidx] == "-auto-top") {
+				auto_top_mode = true;
+				continue;
+			}
 			break;
 		}
 		extra_args(args, argidx, design, false);
@@ -481,35 +502,47 @@ struct HierarchyPass : public Pass {
 
 		log_push();
 
-		if (top_mod == NULL)
+		if (top_mod == nullptr)
 			for (auto &mod_it : design->modules_)
 				if (mod_it.second->get_bool_attribute("\\top"))
 					top_mod = mod_it.second;
 
-		if (top_mod != NULL)
-			hierarchy(design, top_mod, purge_lib, true);
-
-		bool did_something = true;
-		bool did_something_once = false;
-		while (did_something) {
-			did_something = false;
-			std::vector<RTLIL::IdString> modnames;
-			modnames.reserve(design->modules_.size());
-			for (auto &mod_it : design->modules_)
-				modnames.push_back(mod_it.first);
-			for (auto &modname : modnames) {
-				if (design->modules_.count(modname) == 0)
-					continue;
-				if (expand_module(design, design->modules_[modname], flag_check, libdirs))
-					did_something = true;
+		if (top_mod == nullptr && auto_top_mode) {
+			log_header("Finding top of design hierarchy..\n");
+			dict<Module*, int> db;
+			for (Module *mod : design->selected_modules()) {
+				int score = find_top_mod_score(design, mod, db);
+				log("root of %3d design levels: %-20s\n", score, log_id(mod));
+				if (!top_mod || score > db[top_mod])
+					top_mod = mod;
 			}
-			if (did_something)
-				did_something_once = true;
+			if (top_mod != nullptr)
+				log("Automatically selected %s as design top module.\n", log_id(top_mod));
 		}
 
-		if (top_mod != NULL && did_something_once) {
-			log_header("Re-running hierarchy analysis..\n");
-			hierarchy(design, top_mod, purge_lib, false);
+		bool did_something = true;
+		while (did_something)
+		{
+			did_something = false;
+
+			std::set<RTLIL::Module*> used_modules;
+			if (top_mod != NULL) {
+				log_header("Analyzing design hierarchy..\n");
+				hierarchy_worker(design, used_modules, top_mod, 0);
+			} else {
+				for (auto mod : design->modules())
+					used_modules.insert(mod);
+			}
+
+			for (auto module : used_modules) {
+				if (expand_module(design, module, flag_check, libdirs))
+					did_something = true;
+			}
+		}
+
+		if (top_mod != NULL) {
+			log_header("Analyzing design hierarchy..\n");
+			hierarchy_clean(design, top_mod, purge_lib);
 		}
 
 		if (top_mod != NULL) {
@@ -580,5 +613,5 @@ struct HierarchyPass : public Pass {
 		log_pop();
 	}
 } HierarchyPass;
- 
+
 PRIVATE_NAMESPACE_END
