@@ -2,11 +2,11 @@
  *  yosys -- Yosys Open SYnthesis Suite
  *
  *  Copyright (C) 2012  Clifford Wolf <clifford@clifford.at>
- *  
+ *
  *  Permission to use, copy, modify, and/or distribute this software for any
  *  purpose with or without fee is hereby granted, provided that the above
  *  copyright notice and this permission notice appear in all copies.
- *  
+ *
  *  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  *  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  *  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
@@ -79,7 +79,7 @@ struct MemoryShareWorker
 				}
 				return false;
 			}
-				
+
 
 		for (int i = 0; i < int(sig_s.size()); i++)
 		{
@@ -125,6 +125,9 @@ struct MemoryShareWorker
 			created_conditions++;
 		}
 
+		if (terms.size() == 0)
+			terms = State::S1;
+
 		if (terms.size() > 1)
 			terms = module->ReduceAnd(NEW_ID, terms);
 
@@ -207,7 +210,7 @@ struct MemoryShareWorker
 				if (non_feedback_nets.count(sig_data[i]))
 					goto not_pure_feedback_port;
 
-			async_rd_bits[sig_addr].resize(std::max(async_rd_bits.size(), sig_data.size()));
+			async_rd_bits[sig_addr].resize(max(async_rd_bits.size(), sig_data.size()));
 			for (int i = 0; i < int(sig_data.size()); i++)
 				async_rd_bits[sig_addr][i].insert(sig_data[i]);
 
@@ -489,8 +492,8 @@ struct MemoryShareWorker
 		if (wr_ports.size() <= 1)
 			return;
 
-		ezDefaultSAT ez;
-		SatGen satgen(&ez, &modwalker.sigmap);
+		ezSatPtr ez;
+		SatGen satgen(ez.get(), &modwalker.sigmap);
 
 		// find list of considered ports and port pairs
 
@@ -553,7 +556,7 @@ struct MemoryShareWorker
 			if (considered_port_pairs.count(i) || considered_port_pairs.count(i+1))
 			{
 				RTLIL::SigSpec sig = modwalker.sigmap(wr_ports[i]->getPort("\\EN"));
-				port_to_sat_variable[i] = ez.expression(ez.OpOr, satgen.importSigSpec(sig));
+				port_to_sat_variable[i] = ez->expression(ez->OpOr, satgen.importSigSpec(sig));
 
 				std::vector<RTLIL::SigBit> bits = sig;
 				bits_queue.insert(bits.begin(), bits.end());
@@ -582,7 +585,7 @@ struct MemoryShareWorker
 			vector<int> ez_wire_bits = satgen.importSigSpec(wire);
 			for (int i : ez_wire_bits)
 			for (int j : ez_wire_bits)
-				if (i != j) ez.assume(ez.NOT(i), j);
+				if (i != j) ez->assume(ez->NOT(i), j);
 		}
 
 		log("  Common input cone for all EN signals: %d cells.\n", int(sat_cells.size()));
@@ -590,7 +593,7 @@ struct MemoryShareWorker
 		for (auto cell : sat_cells)
 			satgen.importCell(cell);
 
-		log("  Size of unconstrained SAT problem: %d variables, %d clauses\n", ez.numCnfVariables(), ez.numCnfClauses());
+		log("  Size of unconstrained SAT problem: %d variables, %d clauses\n", ez->numCnfVariables(), ez->numCnfClauses());
 
 		// merge subsequent ports if possible
 
@@ -599,13 +602,13 @@ struct MemoryShareWorker
 			if (!considered_port_pairs.count(i))
 				continue;
 
-			if (ez.solve(port_to_sat_variable.at(i-1), port_to_sat_variable.at(i))) {
+			if (ez->solve(port_to_sat_variable.at(i-1), port_to_sat_variable.at(i))) {
 				log("  According to SAT solver sharing of port %d with port %d is not possible.\n", i-1, i);
 				continue;
 			}
 
 			log("  Merging port %d into port %d.\n", i-1, i);
-			port_to_sat_variable.at(i) = ez.OR(port_to_sat_variable.at(i-1), port_to_sat_variable.at(i));
+			port_to_sat_variable.at(i) = ez->OR(port_to_sat_variable.at(i-1), port_to_sat_variable.at(i));
 
 			RTLIL::SigSpec last_addr = wr_ports[i-1]->getPort("\\ADDR");
 			RTLIL::SigSpec last_data = wr_ports[i-1]->getPort("\\DATA");
