@@ -256,7 +256,7 @@ static void create_gold_module(RTLIL::Design *design, RTLIL::IdString cell_type,
 			case 2:
 				n = xorshift32(GetSize(sig));
 				m = xorshift32(GetSize(sig));
-				for (int i = std::min(n, m); i < std::max(n, m); i++)
+				for (int i = min(n, m); i < max(n, m); i++)
 					sig[i] = xorshift32(2) == 1 ? RTLIL::S1 : RTLIL::S0;
 				break;
 			}
@@ -278,10 +278,10 @@ static void run_eval_test(RTLIL::Design *design, bool verbose, bool nosat, std::
 	RTLIL::Module *gate_mod = design->module("\\gate");
 	ConstEval gold_ce(gold_mod), gate_ce(gate_mod);
 
-	ezDefaultSAT ez1, ez2;
+	ezSatPtr ez1, ez2;
 	SigMap sigmap(gold_mod);
-	SatGen satgen1(&ez1, &sigmap);
-	SatGen satgen2(&ez2, &sigmap);
+	SatGen satgen1(ez1.get(), &sigmap);
+	SatGen satgen2(ez2.get(), &sigmap);
 	satgen2.model_undef = true;
 
 	if (!nosat)
@@ -433,7 +433,7 @@ static void run_eval_test(RTLIL::Design *design, bool verbose, bool nosat, std::
 			std::vector<int> sat1_model = satgen1.importSigSpec(out_sig);
 			std::vector<bool> sat1_model_value;
 
-			if (!ez1.solve(sat1_model, sat1_model_value, ez1.vec_eq(sat1_in_sig, sat1_in_val)))
+			if (!ez1->solve(sat1_model, sat1_model_value, ez1->vec_eq(sat1_in_sig, sat1_in_val)))
 				log_error("Evaluating sat model 1 (no undef modeling) failed!\n");
 
 			if (verbose) {
@@ -468,7 +468,7 @@ static void run_eval_test(RTLIL::Design *design, bool verbose, bool nosat, std::
 
 			std::vector<bool> sat2_model_value;
 
-			if (!ez2.solve(sat2_model, sat2_model_value, ez2.vec_eq(sat2_in_def_sig, sat2_in_def_val), ez2.vec_eq(sat2_in_undef_sig, sat2_in_undef_val)))
+			if (!ez2->solve(sat2_model, sat2_model_value, ez2->vec_eq(sat2_in_def_sig, sat2_in_def_val), ez2->vec_eq(sat2_in_undef_sig, sat2_in_undef_val)))
 				log_error("Evaluating sat model 2 (undef modeling) failed!\n");
 
 			if (verbose) {
@@ -536,6 +536,9 @@ struct TestCellPass : public Pass {
 		log("    -simlib\n");
 		log("        use \"techmap -map +/simlib.v -max_iter 2 -autoproc\"\n");
 		log("\n");
+		log("    -aigmap\n");
+		log("        instead of calling \"techmap\", call \"aigmap\"\n");
+		log("\n");
 		log("    -muxdiv\n");
 		log("        when creating test benches with dividers, create an additional mux\n");
 		log("        to mask out the division-by-zero case\n");
@@ -549,11 +552,14 @@ struct TestCellPass : public Pass {
 		log("    -nosat\n");
 		log("        do not check SAT model or run SAT equivalence checking\n");
 		log("\n");
+		log("    -noeval\n");
+		log("        do not check const-eval models\n");
+		log("\n");
 		log("    -v\n");
 		log("        print additional debug information to the console\n");
 		log("\n");
 		log("    -vlog {filename}\n");
-		log("        create a verilog test bench to test simlib and write_verilog\n");
+		log("        create a Verilog test bench to test simlib and write_verilog\n");
 		log("\n");
 	}
 	virtual void execute(std::vector<std::string> args, RTLIL::Design*)
@@ -567,6 +573,7 @@ struct TestCellPass : public Pass {
 		bool verbose = false;
 		bool constmode = false;
 		bool nosat = false;
+		bool noeval = false;
 
 		int argidx;
 		for (argidx = 1; argidx < GetSize(args); argidx++)
@@ -600,6 +607,10 @@ struct TestCellPass : public Pass {
 				techmap_cmd = "techmap -map +/simlib.v -max_iter 2 -autoproc";
 				continue;
 			}
+			if (args[argidx] == "-aigmap") {
+				techmap_cmd = "aigmap";
+				continue;
+			}
 			if (args[argidx] == "-muxdiv") {
 				muxdiv = true;
 				continue;
@@ -612,6 +623,10 @@ struct TestCellPass : public Pass {
 				nosat = true;
 				continue;
 			}
+			if (args[argidx] == "-noeval") {
+				noeval = true;
+				continue;
+			}
 			if (args[argidx] == "-v") {
 				verbose = true;
 				continue;
@@ -765,7 +780,8 @@ struct TestCellPass : public Pass {
 						Backend::backend_call(design, &vlog_file, "<test_cell -vlog>", "verilog -selected -noexpr");
 						uut_names.push_back(uut_name);
 					}
-					run_eval_test(design, verbose, nosat, uut_name, vlog_file);
+					if (!noeval)
+						run_eval_test(design, verbose, nosat, uut_name, vlog_file);
 				}
 				delete design;
 			}
